@@ -427,11 +427,13 @@ def convert_to_ai_sdk_messages(
         "tool_call_end",
         "tool_result",
         "finish_reason",
+        "error",
     ],
 ) -> str:
     """
     Format text events for the AI SDK stream protocol.
-    See: https://ai-sdk.dev/docs/ai-sdk-ui/stream-protocol
+    This follows the data-stream v1 protocol
+    See: https://v4.ai-sdk.dev/docs/ai-sdk-ui/stream-protocol
     """
     TEXT_PREFIX = "0:"
     REASON_PREFIX = "g:"
@@ -441,6 +443,7 @@ def convert_to_ai_sdk_messages(
     TOOL_CALL_PREFIX = "9:"
     TOOL_RESULT_PREFIX = "a:"
     FINISH_REASON_PREFIX = "d:"
+    ERROR_PREFIX = "3:"
 
     # Text events
     if content_type == "text" and isinstance(content_text, str):
@@ -472,6 +475,10 @@ def convert_to_ai_sdk_messages(
     ):
         return f"{REASON_SIGNATURE_PREFIX}{json.dumps(content_text)}\n"
 
+    # Error events
+    elif content_type == "error" and isinstance(content_text, str):
+        return f"{ERROR_PREFIX}{json.dumps(content_text)}\n"
+
     else:
         # Default to text for unknown types
         return f"{TEXT_PREFIX}{json.dumps(content_text)}\n"
